<?php
class GO_Cpt
{

	var $id_base;
	var $post_type_name;
	var $post_meta_key;
	var $post_type_definition;

	function __construct( $cpt_name = FALSE , $cpt_definition = FALSE )
	{
		// set base IDs and such
		$this->id_base = $this->post_type_name = $this->post_meta_key = $cpt_name;

		// save the post definition
		$this->post_type_definition = $cpt_definition;

		// filter post saves to set postmeta from metabox entries
		add_action( 'save_post', array( $this , '_save_post' ));

		// now actually register the post type
		$this->_register_post_type();
	}

	/**
	 * Registers our post type
	 *
	 * @uses register_post_type()
	 *
	 */
	function _register_post_type()
	{
		// automatically add a callback to register a metabox if none is set and the child class has a metabox method
		if(
			! isset( $this->post_type_definition['register_meta_box_cb'] ) &&
			is_callable( array( $this , 'metabox' ))
		)
		{
			$this->post_type_definition['register_meta_box_cb'] = array( $this , '_metaboxes' );
		}

		// register this post type with WP
		// full docs http://codex.wordpress.org/Function_Reference/register_post_type
		register_post_type( $this->post_type_name , $this->post_type_definition );
	}

	/**
	 * Registers our metabox
	 *
	 * This is the function that gets called to tell WP what metaboxes we have, 
	 * how to display them, and what function to call to get the form inside them.
	 *
	 * @uses add_meta_box()
	 *
	 */
	function _metaboxes()
	{
		// register the default metabox for this post type
		// full docs http://codex.wordpress.org/Function_Reference/add_meta_box
		add_meta_box(
			$this->get_field_id('metabox') , // The HTML ID of the metabox
			sprintf( __('%s Settings') , ( isset( $this->post_type_definition['labels']['singular_name'] ) ? $this->post_type_definition['labels']['singular_name'] : $this->post_type_name )) , // The title of the metabox
			array( $this , '_metabox' ) , // the callback function to render the metabox
			$this->post_type_name , // the type of post this metabox is for
			'normal', // in which column to render the metabox
			'high' // where in that column to render it
		);
	}

	/**
	 * The metabox callback function
	 *
	 * This function is inserts the nonce field and calls the child class' metabox() method to display the form inside the metabox
	 *
	 */
	function _metabox( $post )
	{
		$this->nonce_field();
		$this->metabox( $post , $this->get_meta( $post->ID ));
	}

	/**
	 * Inserts a nonce in the metabox
	 *
	 * This function is called internally before the child class' metabox() method is called
	 *
	 * @uses wp_nonce_field()
	 *
	 */
	function nonce_field()
	{
		wp_nonce_field( $this->id_base , $this->id_base .'-nonce' );
	}

	/**
	 * Verifies the nonce
	 *
	 * This function is called internally before the child class' update() method is called
	 *
	 * @uses wp_verify_nonce()
	 *
	 */
	function verify_nonce()
	{
		return wp_verify_nonce( $_POST[ $this->id_base .'-nonce' ] , $this->id_base );
	}


	/**
	 * Constructs name attributes for use in metabox() fields
	 *
	 * This function should be used in metabox() methods to create name attributes for fields to be saved by update()
	 *
	 * @param string $field_name Field name
	 * @return string Name attribute for $field_name
	 */
	function get_field_name( $field_name )
	{
		return $this->id_base . '[' . $field_name . ']';
	}

	/**
	 * Constructs id attributes for use in metabox() fields
	 *
	 * This function should be used in metabox() methods to create id attributes for fields to be saved by update()
	 *
	 * @param string $field_name Field name
	 * @return string ID attribute for $field_name
	 */
	function get_field_id( $field_name )
	{
		return $this->id_base . '-' . $field_name;
	}

	/**
	 * Retrieve metadata for the specified object.
	 *
	 * @uses get_post_meta()
	 *
	 * @param int $object_id ID of the object metadata is for
	 * @return string|array Single metadata value, or array of values
	 */
	function get_meta( $post_id )
	{
		return get_post_meta( $post_id , $this->post_meta_key , TRUE );
	}

	/**
	 * Update metadata for the specified object. Calls the child class' update() method to sanitize the data. 
	 * If no meta value exists for the specified object ID, the metadata will be added.
	 *
	 * @uses update_post_meta()
	 *
	 * @param int $object_id ID of the object metadata is for
	 * @param string $meta_value Metadata value
	 * @return bool True on successful update, false on failure.
	 */
	function update_meta( $post_id , $meta )
	{
		// make sure meta is added to the post, not a revision
		if ( $_post_id = wp_is_post_revision( $post_id ))
			$post_id = $_post_id;

		// apply sanitization from the child class if we have an update method defined
		if( is_callable( array( $this , 'update' )))
		{
			$meta = $this->update( $meta , $this->get_meta( $post_id ));
		}

		update_post_meta( $post_id , $this->post_meta_key , $meta );
	}

	/**
	 * Watch for posts being saved and save our meta data if appropriate
	 *
	 * This is the function is called by the save_post hook. 
	 * After doing many checks, it calls our update_meta() method.
	 *
	 */
	function _save_post( $post_id )
	{

		// don't bother if we don't have an update method defined
		if( ! is_callable( array( $this , 'update' )))
			return;

		// check that this isn't an autosave
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) 
			return;

		// don't run on post revisions (almost always happens just before the real post is saved)
		if( wp_is_post_revision( $post_id ))
			return;

		// check the nonce
		if( ! $this->verify_nonce() )
			return;

		// check the permissions
		// @TODO: make this work with custom permissions
		if( ! current_user_can( 'edit_post' , $post_id ))
			return;	

		// save the meta
		$this->update_meta( $post_id , stripslashes_deep( $_POST[ $this->id_base ] ) );
	}
}